@@ -1,4 +1,4 @@
-/*  $Id: expire.c,v 1.70 1999/12/09 16:29:26 kondou Exp $
+/*  $Id: expire.c,v 1.70.2.1 2001/03/15 23:00:53 kondou Exp $
 **
 **  Expire news articles.
 */
@@ -637,7 +637,11 @@ STATIC BOOL EXPdoline(FILE *out, char *line, int length)
     if (out == NULL)
 	return TRUE;
 
+#ifdef DO_TAGGED_HASH
+    if (ferror(out) || fflush(out) == EOF) {
+#else
     if (ferror(out)) {
+#endif
 	(void)fprintf(stderr, "Can't write new history, %s\n",
 		strerror(errno));
 	return FALSE;
