@@ -1,16 +1,17 @@
-/*  $Id: ng.c 9911 2015-07-04 21:32:56Z iulius $
+/*  $Id: ng.c,v 1.21 1999/11/19 08:59:20 rra Exp $
 **
 **  Routine for the in-core data structures for the active and newsfeeds
 **  files.
 */
-
-#include "config.h"
-#include "clibrary.h"
+#include <stdio.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <sys/mman.h>
 #include <dirent.h>
-
-#include "inn/innconf.h"
+#include "configdata.h"
+#include "clibrary.h"
 #include "innd.h"
-#include "inn/ov.h"
+#include "ov.h"
 
 
 /*
@@ -39,25 +40,43 @@ typedef struct _NGHASH {
 } NGHASH;
 
 
-static struct buffer	NGnames;
-static NGHASH		NGHtable[NGH_SIZE];
-static int		NGHbuckets;
-static int		NGHcount;
-
+STATIC BUFFER	NGdirs;
+STATIC BUFFER	NGnames;
+STATIC NGHASH	NGHtable[NGH_SIZE];
+STATIC int	NGHbuckets;
+STATIC int	NGHcount;
 
+
 
 /*
 **  Sorting predicate for qsort call in NGparsefile.  Put newsgroups in
 **  rough order of their activity.  Will be better if we write a "counts"
 **  file sometime.
 */
-static int
-NGcompare(const void *p1, const void *p2)
+STATIC int
+NGcompare(p1, p2)
+    CPOINTER p1;
+    CPOINTER p2;
 {
-    const NEWSGROUP *const *n1 = p1;
-    const NEWSGROUP *const *n2 = p2;
+    NEWSGROUP	**ng1;
+    NEWSGROUP	**ng2;
 
-    return n1[0]->Last - n2[0]->Last;
+    ng1 = (NEWSGROUP **) p1;
+    ng2 = (NEWSGROUP **) p2;
+    return ng1[0]->Last - ng2[0]->Last;
+}
+
+
+/*
+**  Convert a newsgroup name into a directory name.
+*/
+STATIC void
+NGdirname(p)
+    register char	*p;
+{
+    for ( ; *p; p++)
+	if (*p == '.')
+	    *p = '/';
 }
 
 
@@ -66,62 +85,68 @@ NGcompare(const void *p1, const void *p2)
 **  not to write NUL's into the in-core copy, since we're either mmap(2)'d,
 **  or we want to just blat it out to disk later.
 */
-static bool
-NGparseentry(NEWSGROUP *ngp, const char *p, char *end)
+STATIC BOOL NGparseentry(NEWSGROUP *ngp, char *p, char *end)
 {
-    char		*q;
-    unsigned int	j;
-    NGHASH		*htp;
-    NEWSGROUP		**ngpp;
-    int			i;
-    ARTNUM		lo;
+    register char		*q;
+    register unsigned int	j;
+    register NGHASH		*htp;
+    register NEWSGROUP		**ngpp;
+    register int		i;
+    ARTNUM			lo;
 
     if ((q = strchr(p, ' ')) == NULL)
-	return false;
+	return FALSE;
     i = q - p;
 
     ngp->NameLength = i;
-    ngp->Name = &NGnames.data[NGnames.used];
-    strncpy(ngp->Name, p, i);
+    ngp->Name = &NGnames.Data[NGnames.Used];
+    (void)strncpy(ngp->Name, p, (SIZE_T)i);
     ngp->Name[i] = '\0';
-    NGnames.used += i + 1;
+    NGnames.Used += i + 1;
+
+    ngp->Dir = &NGdirs.Data[NGdirs.Used];
+    (void)strncpy(ngp->Dir, p, (SIZE_T)i);
+    ngp->Dir[i] = '\0';
+    NGdirs.Used += i + 1;
+    NGdirname(ngp->Dir);
 
     ngp->LastString = ++q;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return false;
+	return FALSE;
     ngp->Lastwidth = q - ngp->LastString;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return false;
+	return FALSE;
     lo = (ARTNUM)atol(q + 1);
     if ((q = strchr(q + 1, ' ')) == NULL || q > end)
-	return false;
+	return FALSE;
     ngp->Rest = ++q;
     /* We count on atoi() to stop at the space after the digits! */
     ngp->Last = atol(ngp->LastString);
     ngp->nSites = 0;
-    ngp->Sites = xmalloc(NGHcount * sizeof(int));
+    ngp->Sites = NEW(int, NGHcount);
     ngp->nPoison = 0;
-    ngp->Poison = xmalloc(NGHcount * sizeof(int));
+    ngp->Poison = NEW(int, NGHcount);
     ngp->Alias = NULL;
 
     /* Find the right bucket for the group, make sure there is room. */
+    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(ngp->Name, p, j);
     htp = NGH_BUCKET(j);
     for (p = ngp->Name, ngpp = htp->Groups, i = htp->Used; --i >= 0; ngpp++)
-	if (*p == ngpp[0]->Name[0] && strcmp(p, ngpp[0]->Name) == 0) {
+	if (*p == ngpp[0]->Name[0] && EQ(p, ngpp[0]->Name)) {
 	    syslog(L_ERROR, "%s duplicate_group %s", LogName, p);
-	    return false;
+	    return FALSE;
 	}
     if (htp->Used >= htp->Size) {
 	htp->Size += NGHbuckets;
-        htp->Groups = xrealloc(htp->Groups, htp->Size * sizeof(NEWSGROUP *));
+	RENEW(htp->Groups, NEWSGROUP*, htp->Size);
     }
     htp->Groups[htp->Used++] = ngp;
 
     if (innconf->enableoverview && !OVgroupadd(ngp->Name, lo, ngp->Last, ngp->Rest))
-	return false;
+	return FALSE;
 
-    return true;
+    return TRUE;
 }
 
 
@@ -129,17 +154,17 @@ NGparseentry(NEWSGROUP *ngp, const char *p, char *end)
 **  Parse the active file, building the initial Groups global.
 */
 void
-NGparsefile(void)
+NGparsefile()
 {
-    char	*p;
-    char	*q;
-    int		i;
-    bool	SawMe;
-    NEWSGROUP	*ngp;
-    NGHASH	*htp;
-    char	**strings;
-    char	*active;
-    char	*end;
+    register char	*p;
+    register char	*q;
+    register int	i;
+    register BOOL	SawMe;
+    register NEWSGROUP	*ngp;
+    register NGHASH	*htp;
+    char		**strings;
+    char		*active;
+    char		*end;
 
     /* If re-reading, remove anything we might have had. */
     NGclose();
@@ -152,16 +177,19 @@ NGparsefile(void)
 	syslog(L_FATAL, "%s empty active file", LogName);
 	exit(1);
     }
-    Groups = xmalloc(nGroups * sizeof(NEWSGROUP));
-    GroupPointers = xmalloc(nGroups * sizeof(NEWSGROUP *));
+    Groups = NEW(NEWSGROUP, nGroups);
+    GroupPointers = NEW(NEWSGROUP*, nGroups);
 
-    /* Get space to hold copies of the names.  This might take more space
-     * than individually allocating each element, but it is definitely easier
-     * on the system. */
+    /* Get space to hold copies of the names and the directory names.
+     * This might take more space than individually allocating each
+     * element, but it is definitely easier on the system. */
     i = end - active;
-    NGnames.size = i;
-    NGnames.data = xmalloc(NGnames.size + 1);
-    NGnames.used = 0;
+    NGdirs.Size = i;
+    NGdirs.Data = NEW(char, NGdirs.Size + 1);
+    NGdirs.Used = 0;
+    NGnames.Size = i;
+    NGnames.Data = NEW(char, NGnames.Size + 1);
+    NGnames.Used = 0;
 
     /* Set up the default hash buckets. */
     NGHbuckets = nGroups / NGH_SIZE;
@@ -173,15 +201,15 @@ NGparsefile(void)
     else
 	for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
 	    htp->Size = NGHbuckets;
-	    htp->Groups = xmalloc(htp->Size * sizeof(NEWSGROUP *));
+	    htp->Groups = NEW(NEWSGROUP*, htp->Size);
 	    htp->Used = 0;
 	}
 
     /* Count the number of sites. */
-    SawMe = false;
-    for (strings = SITEreadfile(true), i = 0; (p = strings[i]) != NULL; i++)
+    SawMe = FALSE;
+    for (strings = SITEreadfile(TRUE), i = 0; (p = strings[i]) != NULL; i++)
 	if (*p == 'M' && *++p == 'E' && *++p == ':')
-	    SawMe = true;
+	    SawMe = TRUE;
     if (i == 0 || (i == 1 && SawMe)) {
 	syslog(L_ERROR, "%s bad_newsfeeds no feeding sites", LogName);
 	NGHcount = 1;
@@ -202,7 +230,8 @@ NGparsefile(void)
     /* Sort each bucket. */
     for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++)
 	if (htp->Used > 1)
-	    qsort(htp->Groups, htp->Used, sizeof htp->Groups[0], NGcompare);
+	    qsort((POINTER)htp->Groups, (SIZE_T)htp->Used,
+		sizeof htp->Groups[0], NGcompare);
 
     /* Chase down any alias flags. */
     for (ngp = Groups, i = nGroups; --i >= 0; ngp++)
@@ -223,27 +252,28 @@ NGparsefile(void)
 ** Free allocated memory
 */
 void
-NGclose(void)
+NGclose()
 {
-    int		i;
-    NEWSGROUP	*ngp;
-    NGHASH	*htp;
+    register int	i;
+    register NEWSGROUP	*ngp;
+    register NGHASH	*htp;
 
     if (Groups) {
 	for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
-	    free(ngp->Sites);
-	    free(ngp->Poison);
+	    DISPOSE(ngp->Sites);
+	    DISPOSE(ngp->Poison);
 	}
-	free(Groups);
+	DISPOSE(Groups);
 	Groups = NULL;
-	free(GroupPointers);
-	free(NGnames.data);
+	DISPOSE(GroupPointers);
+	DISPOSE(NGdirs.Data);
+	DISPOSE(NGnames.Data);
     }
 
     for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
       htp->Size = NGHbuckets;
       if (htp->Groups) {
-	free(htp->Groups);
+	DISPOSE(htp->Groups);
 	htp->Used = 0;
 	htp->Groups = NULL;
       }
@@ -254,82 +284,78 @@ NGclose(void)
 **  Hash a newsgroup and see if we get it.
 */
 NEWSGROUP *
-NGfind(const char *Name)
+NGfind(Name)
+    char			*Name;
 {
-    const char		*p;
-    int			i;
-    unsigned int	j;
-    NEWSGROUP		**ngp;
-    char		c;
-    NGHASH		*htp;
+    register char		*p;
+    register int		i;
+    register unsigned int	j;
+    register NEWSGROUP		**ngp;
+    char			c;
+    NGHASH			*htp;
 
+    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(Name, p, j);
     htp = NGH_BUCKET(j);
     for (c = *Name, ngp = htp->Groups, i = htp->Used; --i >= 0; ngp++)
-	if (c == ngp[0]->Name[0] && strcmp(Name, ngp[0]->Name) == 0)
+	if (c == ngp[0]->Name[0] && EQ(Name, ngp[0]->Name))
 	    return ngp[0];
     return NULL;
 }
 
 
 /*
-**  Split a newsgroups header line into the groups we get.  Return the
-**  number of newsgroups.  ' ' and '\t' are dropped when copying.
+**  Split a newsgroups header line into the groups we get.  Return a
+**  point to static memory and clobber the argument along the way.
 */
-int
-NGsplit(char *p, int size, LISTBUFFER *list)
+char **
+NGsplit(p)
+    register char	*p;
 {
-  char		**gp, *q;
-  int		i;
+    static char		**groups;
+    static int		oldlength;
+    register char	**gp;
+    register int	i;
 
-  /* setup buffer */
-  SetupListBuffer(size, list);
-
-  /* loop over and copy */
-  for (i = 0, q = list->Data, gp = list->List ; *p ; p++, *q++ = '\0') {
-    /* skip leading separators and white spaces. */
-    for (; *p && (NG_ISSEP(*p) || ISWHITE(*p)) ; p++)
-      continue;
-    if (*p == '\0')
-      break;
-
-    if (i == list->ListLength) {
-      list->ListLength += DEFAULTNGBOXSIZE;
-      list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
-      gp = &list->List[i];
+    /* Get an array of character pointers. */
+    i = strlen(p);
+    if (groups == NULL) {
+	groups = NEW(char*, i + 1);
+	oldlength = i;
     }
-    /* mark the start of the newsgroup, move to the end of it while copying */
-    for (*gp++ = q, i++ ; *p && !NG_ISSEP(*p) && !ISWHITE(*p) ;) {
-      if (*p == ':')
-	/* reject if ':' is included */
-	return 0;
-      *q++ = *p++;
-      continue;
+    else if (oldlength < i) {
+	RENEW(groups, char*, i + 1);
+	oldlength = i;
     }
-    if (*p == '\0')
-      break;
-  }
-  *q = '\0';
-  if (i == list->ListLength) {
-    list->ListLength += DEFAULTNGBOXSIZE;
-    list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
-    gp = &list->List[i];
-  }
-  *gp = NULL;
-  return i;
+
+    /* Loop over text. */
+    for (gp = groups; *p; *p++ = '\0') {
+	/* Skip leading separators. */
+	for (; NG_ISSEP(*p); p++)
+	    continue;
+	if (*p == '\0')
+	    break;
+
+	/* Mark the start of the newsgroup, move to the end of it. */
+	for (*gp++ = p; *p && !NG_ISSEP(*p); p++)
+	    continue;
+	if (*p == '\0')
+	    break;
+    }
+    *gp = NULL;
+    return groups;
 }
 
 
 /*
 **  Renumber a group.
 */
-#define NORENUMBER "%s cant renumber %s %s too wide"
-#define RENUMBER "%s renumber %s %s from %ld to %ld"
+static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
+static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
 
-bool
-NGrenumber(NEWSGROUP *ngp)
+BOOL NGrenumber(NEWSGROUP *ngp)
 {
-    int			low, high, count, flag;
+    int			low, high, count,flag;
     char	        *f2;
     char		*f3;
     char		*f4;
@@ -338,12 +364,12 @@ NGrenumber(NEWSGROUP *ngp)
     long		l;
     char		*dummy;
 
-    if (!innconf->enableoverview) return true; /* can't do anything w/o overview */
+    if (!innconf->enableoverview) return TRUE; /* can't do anything w/o overview */
 
     /* Get a valid offset into the active file. */
     if (ICDneedsetup) {
 	syslog(L_ERROR, "%s unsynched must reload before renumber", LogName);
-	return false;
+	return FALSE;
     }
     start = ICDreadactive(&dummy) + ngp->Start;
 
@@ -353,57 +379,48 @@ NGrenumber(NEWSGROUP *ngp)
      || (f4 = strchr(++f3, ' ')) == NULL) {
 	syslog(L_ERROR, "%s bad_format active %s",
 	    LogName, MaxLength(start, start));
-	return false;
+	return FALSE;
     }
+    himark = atol(f2);
+    lomark = himark + 1;
+    /* note these will be the low and himarks if the group turns out to be empty. */
 
     /* Check overview data for the group. */
-    if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return false;
+    if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return FALSE;
     if (count != 0) {
-	/* Non-empty group, so set low/high water marks from overview. */
+	/* non-empty group, so set low/himarks from overview. */
 	lomark = low;
 	himark = high;
-    } else {
-        /* Empty group. */
-        himark = high;
-        lomark = himark + 1;
     }
     l = atol(f2);
-    if (himark > l) {
+    if (himark != l) {
 	syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, himark);
-        /* f3 - f2 - 1 gives the length of the full high water mark
-         * in the active file. */
 	if (!FormatLong(f2, himark, f3 - f2 - 1)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
-	    return false;
+	    return FALSE;
 	}
 	ngp->Last = himark;
 	ICDactivedirty++;
-    } else if (himark < l) {
-        syslog(L_NOTICE, "%s renumber %s hi not decreasing %ld to %ld",
-               LogName, ngp->Name, l, himark);
     }
     l = atol(f3);
     if (lomark != l) {
 	if (lomark < l)
 	    syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
-        /* f4 - f3 gives the length of the full low water mark
-         * in the active file.
-         * f4 points at the space ' ' before the newsgroup flag. */
 	if (!FormatLong(f3, lomark, f4 - f3)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-	    return false;
+	    return FALSE;
 	}
 	ICDactivedirty++;
     }
-    return true;
+    return TRUE;
 }
 
 /*
-** Set the low article count for the given group.
-** Like NGrenumber(), but we don't query the overview database.
-*/
-bool
-NGlowmark(NEWSGROUP *ngp, long lomark)
+ * Set the low article count for the given group.
+ * Like NGrenumber(), but we don't scan the spool,
+ * and the himark is ignored.
+ */
+BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
 {
     long l;
     char *f2, *f3, *f4;
@@ -416,31 +433,17 @@ NGlowmark(NEWSGROUP *ngp, long lomark)
      || (f4 = strchr(++f3, ' ')) == NULL) {
         syslog(L_ERROR, "%s bad_format active %s",
             LogName, MaxLength(start, start));
-        return false;
+        return FALSE;
     }
-    /* Check whether the high water mark is more than the low water mark
-     * minus one (in the case of an empty newsgroup).  If not, we consider
-     * that the newsgroup is empty and we increase the high water mark. */
-    l = atol(f2);
-    if (l+1 < lomark) {
-        syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, lomark-1);
-        if (!FormatLong(f2, lomark-1, f3 - f2 - 1)) {
-            syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
-            return false;
-        }
-        ngp->Last = lomark-1;
-        ICDactivedirty++;
-    }
-    /* Update the low water mark. */
     l = atol(f3);
     if (lomark != l) {
         if (lomark < l)
             syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
         if (!FormatLong(f3, lomark, f4 - f3)) {
             syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-            return false;
+            return FALSE;
         }
         ICDactivedirty++;
     }
-    return true;
+    return TRUE;
 }
