@@ -1,5 +1,5 @@
 /*
-**  $Id: python.c,v 1.5 1999/10/24 09:14:51 kondou Exp $
+**  $Id: python.c,v 1.5.2.1 2000/12/10 12:55:58 kondou Exp $
 **
 **  python.c: Embed Python in the style of innd's TCL and Perl stuff.
 ** 
@@ -139,10 +139,11 @@ PYartfilter(artBody, artLen, lines)
     }
 
     /* ...then the body... */
-    if (artBody != NULL) {
-	PYheaditem[hdrnum] = PyBuffer_FromMemory(artBody, --artLen);
-	PyDict_SetItem(PYheaders, PYbodykey, PYheaditem[hdrnum++]);
-    }
+    if (artLen && artBody != NULL)
+        PYheaditem[hdrnum] = PyBuffer_FromMemory(artBody, --artLen);
+    else
+        PYheaditem[hdrnum] = Py_None;
+    PyDict_SetItem(PYheaders, PYbodykey, PYheaditem[hdrnum++]);
 
     /* ...and finally, the line count. */
     PYheaditem[hdrnum] = PyInt_FromLong((long) lines);
