@@ -4,7 +4,7 @@
  * Start Date:  Wed Dec 27 17:10:18 1995
  * Project:     INN (innfeed)
  * File:        misc.c
- * RCSId:       $Id: misc.c,v 1.8.2.3 2000/10/01 22:55:30 kondou Exp $
+ * RCSId:       $Id: misc.c,v 1.8.2.4 2001/04/12 16:20:24 rra Exp $
  *
  * Copyright:   Copyright (c) 1996 by Internet Software Consortium
  *
@@ -29,7 +29,7 @@
  */
 
 #if ! defined (lint)
-static const char *rcsid = "$Id: misc.c,v 1.8.2.3 2000/10/01 22:55:30 kondou Exp $" ;
+static const char *rcsid = "$Id: misc.c,v 1.8.2.4 2001/04/12 16:20:24 rra Exp $" ;
 static void use_rcsid (const char *rid) {   /* Never called */
   use_rcsid (rcsid) ; use_rcsid (rid) ;
 }
@@ -154,10 +154,7 @@ void logOrPrint (int level, FILE *fp, const char *fmt, ...)
     }
   else
     {
-      char buffer [512] ;      /* gag me */
-
-      vsprintf (buffer,fmt,ap) ;
-      syslog (level,buffer) ;
+      dolog (level,fmt,ap) ;
     }
   va_end (ap) ;
 }
