@@ -1,37 +1,31 @@
-/*  $Id: clientactive.c 9496 2013-06-25 17:40:26Z iulius $
+/*  $Revision: 1.7 $
 **
 */
-
-#include "config.h"
+#include <stdio.h>
+#include <sys/types.h>
+#include "configdata.h"
 #include "clibrary.h"
 #include <errno.h>
-
-#include "inn/innconf.h"
-#include "inn/libinn.h"
-#include "inn/nntp.h"
-#include "inn/paths.h"
+#include "paths.h"
+#include "libinn.h"
+#include "nntp.h"
+#include "macros.h"
 
 
-static char	*CApathname;
-static FILE	*CAfp;
+STATIC char	CApathname[256];
+STATIC FILE	*CAfp;
 
 
 /*
 **  Get a copy of the active file for a client host to use, locally or
 **  remotely.
 */
-FILE *
-CAopen(FILE *FromServer, FILE *ToServer)
+FILE *CAopen(FILE *FromServer, FILE *ToServer)
 {
-    char *path;
-
     /* Use a local (or NFS-mounted) copy if available.  Make sure we don't
      * try to delete it when we close it. */
-    path = concatpath(innconf->pathdb, INN_PATH_CLIENTACTIVE);
-    CAfp = fopen(path, "r");
-    free(path);
-    if (CAfp != NULL) {
-	CApathname = NULL;
+    if ((CAfp = fopen(cpcatpath(innconf->pathdb, _PATH_CLIENTACTIVE), "r")) != NULL) {
+	CApathname[0] = '\0';
 	return CAfp;
     }
 
@@ -43,38 +37,31 @@ CAopen(FILE *FromServer, FILE *ToServer)
 /*
 **  Internal library routine.
 */
-FILE *
-CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer,
-            const char *request)
+FILE *CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer, char *request)
 {
     char	buff[BUFSIZ];
-    char        expectedanswer[BUFSIZ];
     char	*p;
     int		oerrno;
     FILE	*F;
 
-    F = fopen(pathname, "w");
-    if (F == NULL)
+    (void)unlink(pathname);
+    if ((F = fopen(pathname, "w")) == NULL)
 	return NULL;
 
-    /* Send a LIST command and capture the output. */
+    /* Send a LIST command to and capture the output. */
     if (request == NULL)
-	fprintf(ToServer, "LIST\r\n");
+	(void)fprintf(ToServer, "list\r\n");
     else
-	fprintf(ToServer, "LIST %s\r\n", request);
-    fflush(ToServer);
-
-    snprintf(expectedanswer, sizeof(expectedanswer), "%d", NNTP_OK_LIST);
+	(void)fprintf(ToServer, "list %s\r\n", request);
+    (void)fflush(ToServer);
 
     /* Get the server's reply to our command. */
     if (fgets(buff, sizeof buff, FromServer) == NULL
-     || strncmp(buff, expectedanswer, strlen(expectedanswer)) != 0) {
+     || !EQn(buff, NNTP_LIST_FOLLOWS, STRLEN(NNTP_LIST_FOLLOWS))) {
 	oerrno = errno;
-	/* Only call CAclose() if opened through CAopen(). */
-	if (strcmp(CApathname, pathname) == 0)
-            CAclose();
+	/* Only call CAclose() if opened through CAopen() */
+	if (strcmp(CApathname, pathname) == 0) CAclose();
 	errno = oerrno;
-        fclose(F);
 	return NULL;
     }
 
@@ -89,12 +76,12 @@ CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer,
 		break;
 	    return fopen(pathname, "r");
 	}
-	fprintf(F, "%s\n", buff);
+	(void)fprintf(F, "%s\n", buff);
     }
 
     /* Ran out of input before finding the terminator; quit. */
     oerrno = errno;
-    fclose(F);
+    (void)fclose(F);
     CAclose();
     errno = oerrno;
     return NULL;
@@ -105,27 +92,14 @@ CA_listopen(char *pathname, FILE *FromServer, FILE *ToServer,
 **  Use the NNTP list command to get a file from a server.  Default is
 **  the active file, otherwise ask for whatever is in the request param.
 */
-FILE *
-CAlistopen(FILE *FromServer, FILE *ToServer, const char *request)
+FILE *CAlistopen(FILE *FromServer, FILE *ToServer, char *request)
 {
-    int fd, oerrno;
-
     /* Gotta talk to the server -- see if we can. */
-    if (FromServer == NULL || ToServer == NULL) {
-	errno = EBADF;
+    if (FromServer == NULL || ToServer == NULL)
 	return NULL;
-    }
 
-    CApathname = concatpath(innconf->pathtmp, INN_PATH_TEMPACTIVE);
-    fd = mkstemp(CApathname);
-    if (fd < 0) {
-        oerrno = errno;
-        free(CApathname);
-        CApathname = 0;
-        errno = oerrno;
-        return NULL;
-    }
-    close(fd);
+    (void)sprintf(CApathname, "%.220s/%s", innconf->pathtmp, _PATH_TEMPACTIVE);
+    (void)mktemp(CApathname);
     return CAfp = CA_listopen(CApathname, FromServer, ToServer, request);
 }
 
@@ -134,15 +108,14 @@ CAlistopen(FILE *FromServer, FILE *ToServer, const char *request)
 /*
 **  Close the file opened by CAopen or CAlistopen.
 */
-void
-CAclose(void)
+void CAclose(void)
 {
     if (CAfp) {
-	fclose(CAfp);
+	(void)fclose(CAfp);
 	CAfp = NULL;
     }
-    if (CApathname != NULL) {
-	unlink(CApathname);
-	CApathname = NULL;
+    if (CApathname[0]) {
+	(void)unlink(CApathname);
+	CApathname[0] = '\0';
     }
 }
