@@ -1,115 +1,88 @@
-/*  $Id: conffile.c 9582 2013-12-09 21:30:42Z iulius $
+/* $Revision: 1.9 $
 **
-**  Routines for reading in incoming.conf-style config files.
+** Routines for reading in incoming.conf-style config files.
 */
-
 #include "config.h"
 #include "clibrary.h"
 #include "conffile.h"
-#include "inn/libinn.h"
-
-static int getconfline(CONFFILE *F, char *buffer, int length) {
-  if (F->f) {
-    fgets(buffer, length, F->f);
-    if (ferror(F->f)) {
-      return 1;
-    }
-  } else if (F->array) {
-    strlcpy(buffer, F->array[F->lineno], F->sbuf);
-  }
-  F->lineno++;
-  if (strlen (F->buf) >= F->sbuf - 1) {
-    return 1; /* Line too long */
-  } else {
-    return 0;
-  }
-}
-
-static int cfeof(CONFFILE *F) {
-  if (F->f) {
-    return feof(F->f);
-  } else if (F->array) {
-    return (F->lineno == F->array_len);
-  } else {
-    return 1;
-  }
-}
+#include "libinn.h"
+#include "macros.h"
 
 static char *CONFgetword(CONFFILE *F)
 {
-  char *p;
-  char *s;
-  char *t;
-  char *word;
-  bool comment;
+  register char *p;
+  register char *s;
+  register char *t;
+  char          *word;
+  register BOOL flag;
 
   if (!F) return (NULL);	/* No conf file */
   if (!F->buf || !F->buf[0]) {
-    if (cfeof (F)) return (NULL);
+    if (feof (F->f)) return (NULL);
     if (!F->buf) {
       F->sbuf = BIG_BUFFER;
-      F->buf = xmalloc(F->sbuf);
+      F->buf = NEW(char, F->sbuf);
     }
-    if (getconfline(F, F->buf, F->sbuf) != 0)
+    fgets(F->buf, F->sbuf, F->f);
+    F->lineno++;
+    if (strlen (F->buf) == F->sbuf)
       return (NULL); /* Line too long */
   }
   do {
      /* Ignore blank and comment lines. */
      if ((p = strchr(F->buf, '\n')) != NULL)
        *p = '\0';
+     if ((p = strchr(F->buf, COMMENT_CHAR)) != NULL) {
+       if (p == F->buf || (p > F->buf && *(p - 1) != '\\'))
+           *p = '\0';
+     }
      for (p = F->buf; *p == ' ' || *p == '\t' ; p++);
-     if ((*p == '\0' || *p == '#') && !cfeof(F)) {
-       if (getconfline(F, F->buf, F->sbuf))
+     flag = TRUE;
+     if (*p == '\0' && !feof (F->f)) {
+       flag = FALSE;
+       fgets(F->buf, F->sbuf, F->f);
+       F->lineno++;
+       if (strlen (F->buf) == F->sbuf)
          return (NULL); /* Line too long */
        continue;
      }
      break;
-  } while (!cfeof(F));
+  } while (!feof (F->f) || !flag);
 
-  comment = false;
   if (*p == '"') { /* double quoted string ? */
     p++;
     do {
       for (t = p; (*t != '"' || (*t == '"' && *(t - 1) == '\\')) &&
              *t != '\0'; t++);
       if (*t == '\0') {
-        if (strlen(F->buf) >= F->sbuf - 2)
-          return (NULL); /* Line too long */
         *t++ = '\n';
-        *t = '\0';
-        if (getconfline(F, t, F->sbuf - strlen(F->buf)))
+        fgets(t, F->sbuf - strlen (F->buf), F->f);
+	F->lineno++;
+        if (strlen (F->buf) == F->sbuf)
           return (NULL); /* Line too long */
         if ((s = strchr(t, '\n')) != NULL)
           *s = '\0';
       }
       else 
         break;
-    } while (!cfeof(F));
-    if (*t != '"')
-      return (NULL);
+    } while (!feof (F->f));
     *t++ = '\0';
   }
   else {
-    for (t = p; *t != ' ' && *t != '\t' && *t != '\0'; t++)
-      if (*t == '#' && (t == p || *(t - 1) != '\\')) {
-        comment = true;
-        break;
-      }
+    for (t = p; *t != ' ' && *t != '\t' && *t != '\0'; t++);
     if (*t != '\0')
       *t++ = '\0';
   }
-  if (*p == '\0' && cfeof(F)) return (NULL);
-  word = xstrdup (p);
-  p = F->buf;
-  if (!comment)
-    for (; *t != '\0'; t++)
-      *p++ = *t;
+  if (*p == '\0' && feof (F->f)) return (NULL);
+  word = COPY (p);
+  for (p = F->buf; *t != '\0'; t++)
+    *p++ = *t;
   *p = '\0';
 
   return (word);
 }
 
-CONFFILE *CONFfopen(const char *filename)
+CONFFILE *CONFfopen(char *filename)
 {
   FILE *f;
   CONFFILE *ret;
@@ -117,17 +90,16 @@ CONFFILE *CONFfopen(const char *filename)
   f = fopen(filename, "r");
   if (!f)
     return(0);
-  ret = xmalloc(sizeof(CONFFILE));
+  ret = NEW(CONFFILE, 1);
   if (!ret) {
     fclose(f);
     return(0);
   }
-  ret->filename = xstrdup(filename);
+  ret->filename = COPY(filename);
   ret->buf = 0;
   ret->sbuf = 0;
   ret->lineno = 0;
   ret->f = f;
-  ret->array = NULL;
   return(ret);
 }
 
@@ -136,10 +108,10 @@ void CONFfclose(CONFFILE *f)
   if (!f) return;		/* No conf file */
   fclose(f->f);
   if (f->buf)
-    free(f->buf);
+    DISPOSE(f->buf);
   if (f->filename)
-    free(f->filename);
-  free(f);
+    DISPOSE(f->filename);
+  DISPOSE(f);
 }
 
 CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
@@ -149,7 +121,7 @@ CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
   int i;
 
   if (ret.name) {
-    free(ret.name);
+    DISPOSE(ret.name);
     ret.name = 0;
   }
   word = CONFgetword(file);
@@ -157,8 +129,8 @@ CONFTOKEN *CONFgettoken(CONFTOKEN *toklist, CONFFILE *file)
     return(0);
   if (toklist) {
     for (i = 0; toklist[i].type; i++) {
-       if (strcmp(word, toklist[i].name) == 0) {
-         free(word);
+       if (EQ(word, toklist[i].name)) {
+         DISPOSE(word);
          return(&toklist[i]);
        }
     }
