@@ -164,12 +164,6 @@ static int canttag_warned;		/* flag to control can't tag warning */
 
 #endif	/* DO_TAGGED_HASH */
 
-#ifdef MAP_FILE
-#define MAP__ARG	(MAP_FILE | MAP_SHARED)
-#else
-#define MAP__ARG	(MAP_SHARED)
-#endif
-
 /* Old dbz used a long as the record type for dbz entries, which became
  * really gross in places because of mixed references.  We define these to
  * make it a bit easier if we want to store more in here.
@@ -791,10 +785,6 @@ int dbzinit(const char *name) {
     tagboth = tagbits | taghere;
     canttag_warned = 0;
 #else
-    if ((innconf == NULL) && (ReadInnConf() < 0)) {
-	Fclose(dirf);
-	return FALSE;
-    }
     if (!openhashtable(name, idx, &idxtab, sizeof(of_t), options.pag_incore)) {
 	Fclose(dirf);
 	return FALSE;
@@ -1390,7 +1380,7 @@ static of_t			/* NOTFOUND if we hit VACANT or error */
 search(searcher *sp)
 {
     of_t value;
-    unsigned long taboffset = 0;
+    unsigned long taboffset = sp->tabno * conf.tsize;
 
     if (sp->aborted)
 	return(NOTFOUND);
@@ -1423,10 +1413,9 @@ search(searcher *sp)
 		    return(NOTFOUND);
 		} else
 		    value = VACANT;
-	    }
-
-	    /* and finish up */
-	    pagtab.pos += sizeof(value);
+                pagtab.pos = -1;
+	    } else
+                pagtab.pos += sizeof(value);
 	}
 
 	/* vacant slot is always cause to return */
