@@ -2,15 +2,16 @@
    is implemented using MD5, but should probably have a mechanism for
    choosing the hash algorithm and tagging the hash with the algorithm
    used */
-#include "config.h"
-#include "clibrary.h"
+#include <stdio.h>
+#include <sys/types.h>
 #include <ctype.h>
+#include "configdata.h"
+#include "clibrary.h"
+#include "libinn.h"
+#include "macros.h"
+#include "md5.h"
 
-#include "inn/md5.h"
-#include "inn/utility.h"
-#include "inn/libinn.h"
-
-static HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
+STATIC HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
 		       0, 0, 0, 0, 0, 0, 0, 0 }};
 
 /* cipoint - where in this message-ID does it become case-insensitive?
@@ -26,15 +27,13 @@ static HASH empty= { { 0, 0, 0, 0, 0, 0, 0, 0,
  *
  * Returns: pointer into s, or NULL for "nowhere"
  */
-static const char *
-cipoint(const char *s, size_t size)
-{
+STATIC char *cipoint(char *s, size_t size) {
     char *p;
-    static const char post[] = "postmaster";
+    static char post[] = "postmaster";
     static int plen = sizeof(post) - 1;
 
-    if ((p = memchr(s, '@', size))== NULL)	/* no local/domain split */
-	return NULL;				/* assume all local */
+    if ((p = memchr(s, '@', size))== NULL)			/* no local/domain split */
+	return NULL;		/* assume all local */
     if ((p - (s + 1) == plen) && !strncasecmp(post, s+1, plen)) {
 	/* crazy -- "postmaster" is case-insensitive */
 	return s;
@@ -42,46 +41,36 @@ cipoint(const char *s, size_t size)
     return p;
 }
 
-HASH
-Hash(const void *value, const size_t len)
-{
-    struct md5_context context;
+HASH Hash(const void *value, const size_t len) {
+    MD5_CTX context;
     HASH hash;
 
-    md5_init(&context);
-    md5_update(&context, value, len);
-    md5_final(&context);
+    MD5Init(&context);
+    MD5Update(&context, value, len);
+    MD5COUNT(&context, len);
+    MD5Final(&context);
     memcpy(&hash,
 	   &context.digest,
 	   (sizeof(hash) < sizeof(context.digest)) ? sizeof(hash) : sizeof(context.digest));
     return hash;
 }
 
-HASH
-HashMessageID(const char *MessageID)
-{
-    char                *new = NULL;
-    const char          *cip, *p = NULL;
-    char                *q;
+HASH HashMessageID(const char *MessageID) {
+    char                *new;
+    char                *cip;
+    char                *p;
     int                 len;
     HASH                hash;
 
     len = strlen(MessageID);
-    cip = cipoint(MessageID, len);
-    if (cip != NULL) {
-        for (p = cip + 1; *p != '\0'; p++) {
-            if (!islower((unsigned char) *p)) {
-                new = xstrdup(MessageID);
-                break;
-            }
-        }
+	    new = NEW(char, len + 1);
+    strcpy(new, MessageID);
+    if ((cip = cipoint(new, len))) {
+	for (p = cip + 1; *p; p++)
+	    *p = tolower(*p);
     }
-    if (new != NULL)
-        for (q = new + (p - MessageID); *q != '\0'; q++)
-            *q = tolower((unsigned char) *q);
-    hash = Hash(new ? new : MessageID, len);
-    if (new != NULL)
-	free(new);
+    hash = Hash(new, len);
+    DISPOSE(new);
     return hash;
 }
 
@@ -89,9 +78,7 @@ HashMessageID(const char *MessageID)
 **  Check if the hash is all zeros, and subseqently empty, see HashClear
 **  for more info on this.
 */
-bool
-HashEmpty(const HASH h)
-{
+BOOL HashEmpty(const HASH h) {
     return (memcmp(&empty, &h, sizeof(HASH)) == 0);
 }
 
@@ -100,36 +87,46 @@ HashEmpty(const HASH h)
 **  introduces the possibility of colliding w/a value that actually hashes
 **  to all zeros, but that's fairly unlikely.
 */
-void
-HashClear(HASH *hash)
-{
+void HashClear(HASH *hash) {
     memset(hash, '\0', sizeof(HASH));
 }
 
 /*
 **  Convert the binary form of the hash to a form that we can use in error
-**  messages and logs.  Just a wrapper around inn_encode_hex.
+**  messages and logs.
 */
-char *
-HashToText(const HASH hash)
-{
-    static char	hashstr[(sizeof(HASH) * 2) + 1];
+char *HashToText(const HASH hash) {
+    STATIC char         hex[] = "0123456789ABCDEF";
+    char                *p;
+    unsigned int        i;
+    STATIC char         hashstr[(sizeof(HASH)*2) + 1];
 
-    inn_encode_hex((unsigned char *) hash.hash, sizeof(HASH), hashstr,
-                   sizeof(hashstr));
+    for (p = (char *)&hash, i = 0; i < sizeof(HASH); i++, p++) {
+	hashstr[i * 2] = hex[(*p & 0xF0) >> 4];
+	hashstr[(i * 2) + 1] = hex[*p & 0x0F];
+    }
+    hashstr[(sizeof(HASH) * 2)] = '\0';
     return hashstr;
 }
 
 /*
-**  Convert the ASCII representation of the hash back to the canonical form.
-**  Just a wrapper around inn_decode_hex.
+** Converts a hex digit and converts it to a int
 */
-HASH
-TextToHash(const char *text)
-{
-    HASH hash;
+STATIC int hextodec(const int c) {
+    return isdigit(c) ? (c - '0') : ((c - 'A') + 10);
+}
 
-    inn_decode_hex(text, (unsigned char *) hash.hash, sizeof(HASH));
+/*
+**  Convert the ASCII representation of the hash back to the canonical form
+*/
+HASH TextToHash(const char *text) {
+    char                *q;
+    int                 i;
+    HASH                hash;
+
+    for (q = (char *)&hash, i = 0; i != sizeof(HASH); i++) {
+	q[i] = (hextodec(text[i * 2]) << 4) + hextodec(text[(i * 2) + 1]);
+    }
     return hash;
 }
 
