@@ -1,4 +1,4 @@
-/*  $Id: commands.c,v 1.40.2.1 2000/06/12 01:26:39 kondou Exp $
+/*  $Id: commands.c,v 1.40.2.2 2000/10/01 22:56:09 kondou Exp $
 **
 **  Miscellaneous commands.
 */
@@ -667,7 +667,7 @@ FUNCTYPE CMDnewgroups(int ac, char *av[])
 	}
 	if (listsize <= numgroups) {
 	    listsize += 1000;
-	    grouplist = RENEW(grouplist, GROUPDATA, listsize);
+	    RENEW(grouplist, GROUPDATA, listsize);
 	}
 
 	grouplist[numgroups].high = hi;
