@@ -1,4 +1,4 @@
-/*  $Revision: 1.78.2.17 $
+/*  $Revision: 1.78.2.20 $
 **
 **  NNTP server for readers (NNRP) for InterNetNews.
 **  This server doesn't do any real load-limiting, except for what has
@@ -486,15 +486,7 @@ STATIC void StartConnection()
 	HostErrorStr = NULL;
 	if (!Address2Name(&sin.sin_addr, ServerHost, sizeof(ServerHost))) {
 	    strcpy(ServerHost, inet_ntoa(sin.sin_addr));
-	    if (HostErrorStr == NULL) {
-		syslog(L_NOTICE,
-		    "? cant gethostbyaddr %s %m -- using IP address for access",
-		    ServerHost);
-	    } else {
-		syslog(L_NOTICE,
-		    "? cant gethostbyaddr %s %s -- using IP address for access",
-		    ServerHost, HostErrorStr);
-	    }
+	    /* suppress error reason */
 	}
 #else
         strcpy(ServerHost, inet_ntoa(sin.sin_addr));
@@ -955,6 +947,9 @@ main(int argc, char *argv[])
 
 	/* Set signal handle to care for dead children */
 	(void)xsignal(SIGCHLD, WaitChild);
+
+	/* Arrange to toggle tracing. */
+	(void)xsignal(SIGHUP, ToggleTrace);
  
 	TITLEset("nnrpd: accepting connections");
  	
@@ -974,6 +969,11 @@ main(int argc, char *argv[])
 		syslog(L_NOTICE, "cant fork (waiting): %m");
 		sleep(1);
 	    }
+	    if (ChangeTrace) {
+		Tracing = Tracing ? FALSE : TRUE;
+		syslog(L_TRACE, "trace %sabled", Tracing ? "en" : "dis");
+		ChangeTrace = FALSE;
+	    }
 	    if (pid != 0)
 		close(fd);
 	} while (pid != 0);
@@ -998,6 +998,8 @@ main(int argc, char *argv[])
  
     } else {
 	SetupDaemon();
+	/* Arrange to toggle tracing. */
+	(void)xsignal(SIGHUP, ToggleTrace);
     }/* DaemonMode */
 
     /* Setup. */
@@ -1033,7 +1035,7 @@ main(int argc, char *argv[])
       ssl_result=tls_start_servertls(0, /* read */
 				     1); /* write */
       if (ssl_result==-1) {
-	Reply("%d Starttls failed\r\n", NNTP_STARTTLS_BAD_VAL);
+	Reply("%d SSL connection failed\r\n", NNTP_STARTTLS_BAD_VAL);
 	ExitWithStats(1, FALSE);
       }
 
@@ -1054,9 +1056,6 @@ main(int argc, char *argv[])
     /* Catch SIGPIPE so that we can exit out of long write loops */
     (void)xsignal(SIGPIPE, CatchPipe);
 
-    /* Arrange to toggle tracing. */
-    (void)xsignal(SIGHUP, ToggleTrace);
-
     /* Get permissions and see if we can talk to this client */
     StartConnection();
     if (!PERMcanread && !PERMcanpost && !PERMneedauth) {
