@@ -3,7 +3,7 @@
 #
 # Sample file tested with INN 2.3, 2.2, 1.7.2 and 1.5.1
 #
-# (c) 1997-1999 by Fabien Tassin <fta@oleane.net>
+# (c) 1997-1999 by Fabien Tassin <fta@sofaraway.org>
 # version 3.0.2
 ##########################################################
 
@@ -164,6 +164,10 @@ sub collect {
       $inn_badart{$server}++;
       return 1;
     }
+    # Cancelling <...>
+    if ($left =~ /(\S+) <[^>]+> Cancelling/o) {
+      return 1;
+    }
     # all others are just counted as "Other"
     if ($left =~ /(\S+) /o) {
       my $server = $1;
@@ -643,6 +647,8 @@ sub collect {
     return 1 if $left =~ /CNFS-sm: cycbuff \S+ rollover to cycle/o;
     # CNFS-sm: CNFSflushallheads: flushing - possibly interesting
     return 1 if $left =~ /CNFS-sm: CNFSflushallheads: flushing /o;
+    # Cleanfeed status reports
+    return 1 if $left =~ /^filter: status/o;
   }
   ########
   ## innfeed
