@@ -1,4 +1,4 @@
-/*  $Id: buffindexed.c,v 1.40.2.4 2000/11/10 01:57:28 kondou Exp $
+/*  $Id: buffindexed.c,v 1.40.2.6 2001/02/12 16:07:58 kondou Exp $
 **
 **  Overview buffer and index method.
 */
@@ -369,10 +369,6 @@ STATIC BOOL ovbuffread_config(void) {
     return FALSE;
   }
   for (from = to = config; *from; ) {
-    if (ctab_free == 0)
-      ctab = NEW(char *, 1);
-    else
-      RENEW(ctab, char *, ctab_free+1);
     if (*from == '#') {	/* Comment line? */
       while (*from && *from != '\n')
 	from++;	/* Skip past it */
@@ -383,6 +379,10 @@ STATIC BOOL ovbuffread_config(void) {
       from++;
       continue;		/* Back to top of loop */
     }
+    if (ctab_free == 0)
+      ctab = NEW(char *, 1);
+    else
+      RENEW(ctab, char *, ctab_free+1);
     /* If we're here, we've got the beginning of a real entry */
     ctab[ctab_free++] = to = from;
     while (1) {
@@ -1799,7 +1799,7 @@ BOOL buffindexed_expiregroup(char *group, int *lo) {
 	if (Gib[j].artnum == 0)
 	  continue;
 	/* this may be duplicated, but ignore it in this case */
-	OVEXPremove(Gib[j].token, TRUE);
+	OVEXPremove(Gib[j].token, TRUE, NULL, 0);
       }
       freegroupblock();
       ovgroupunmap();
