@@ -1,4 +1,4 @@
-/*  $Id: cnfs.c,v 1.73.2.3 2000/12/21 22:18:02 kondou Exp $
+/*  $Id: cnfs.c,v 1.73.2.6 2001/03/29 01:04:13 kondou Exp $
 **
 **  Cyclic News File System.
 */
@@ -639,6 +639,7 @@ STATIC BOOL CNFSinit_disks(CYCBUFF *cycbuff) {
 	cycbuff->currentbuff = TRUE;
 	cycbuff->order = 0;	/* to indicate this is newly added cycbuff */
 	cycbuff->needflush = TRUE;
+	memset(cycbuff->metaname, '\0', CNFSLASIZ);
 	if (!CNFSflushhead(cycbuff))
 	    return FALSE;
     }
@@ -732,10 +733,6 @@ STATIC BOOL CNFSread_config(void) {
 	return FALSE;
     }
     for (from = to = config; *from; ) {
-	if (ctab_free == 0)
-	  ctab = NEW(char *, 1);
-	else
-	  RENEW(ctab, char *, ctab_free+1);
 	if (*from == '#') {	/* Comment line? */
 	    while (*from && *from != '\n')
 		from++;				/* Skip past it */
@@ -746,6 +743,10 @@ STATIC BOOL CNFSread_config(void) {
 	    from++;
 	    continue;				/* Back to top of loop */
 	}
+	if (ctab_free == 0)
+	  ctab = NEW(char *, 1);
+	else
+	  RENEW(ctab, char *, ctab_free+1);
 	/* If we're here, we've got the beginning of a real entry */
 	ctab[ctab_free++] = to = from;
 	while (1) {
@@ -1760,6 +1761,10 @@ BOOL cnfs_flushcacheddata(FLUSHTYPE type) {
     return TRUE; 
 }
 
+void cnfs_printfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
+    fprintf(file, "%s\n", TokenToText(token));
+}
+
 void cnfs_shutdown(void) {
     CNFSflushallheads();
     CNFSmunmapbitfields();
