@@ -1,4 +1,4 @@
-/*  $Id: interface.c,v 1.45.2.4 2000/09/05 23:24:22 kondou Exp $
+/*  $Id: interface.c,v 1.45.2.6 2001/03/26 22:41:43 kondou Exp $
 **
 **  Storage Manager interface
 */
@@ -236,10 +236,14 @@ STATIC char *GetXref(ARTHANDLE *art) {
   if (!Nocr)
     p = p1;
   /* skip pathhost */
-  for (; *q == ' '; q++);
-  if ((q = strchr(q, ' ')) == NULL)
-    return 0;
-  for (q++; *q == ' '; q++);
+  for (; (*q == ' ') && (q < p); q++);
+  if (q == p)
+    return NULL;
+  if ((q = memchr(q, ' ', p - q)) == NULL)
+    return NULL;
+  for (q++; (*q == ' ') && (q < p); q++);
+  if (q == p)
+    return NULL;
   buff = NEW(char, p - q + 1);
   memcpy(buff, q, p - q);
   buff[p - q] = '\0';
@@ -910,6 +914,18 @@ BOOL SMflushcacheddata(FLUSHTYPE type) {
     return TRUE;
 }
 
+void SMprintfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
+    if (method_data[typetoindex[token.type]].initialized == INIT_FAIL)
+	return;
+    if (method_data[typetoindex[token.type]].initialized == INIT_NO
+        && !InitMethod(typetoindex[token.type])) {
+	SMseterror(SMERR_UNINIT, NULL);
+	syslog(L_ERROR, "SM can't print files for article with uninitialized method");
+	return;
+    }
+    storage_methods[typetoindex[token.type]].printfiles(file, token, xref, ngroups);
+}
+
 void SMshutdown(void) {
     int                 i;
     STORAGE_SUB         *old;
