@@ -1,4 +1,4 @@
-/*  $Id: timecaf.c,v 1.10.2.2 2000/09/16 17:11:49 kondou Exp $
+/*  $Id: timecaf.c,v 1.10.2.3 2001/02/03 07:30:12 rra Exp $
 **
 **  Like the timehash storage method (and heavily inspired by it), but uses
 **  the CAF library to store multiple articles in a single file.
@@ -854,6 +854,10 @@ BOOL timecaf_flushcacheddata(FLUSHTYPE type) {
     return TRUE;
 }
 
+void timecaf_printfiles(FILE *file, TOKEN token, char **xref, int ngroups) {
+    fprintf(file, "%s\n", TokenToText(token));
+}
+
 void timecaf_shutdown(void) {
     CloseOpenFile(&WritingFile);
     DoCancels();
